@@ -442,7 +442,38 @@ assayDataElementNames <- function(object) {
 
 assayDataElement <- function(object, elt) assayData(object)[[elt]]
 
-assayDataElementReplace <- function(obj, elt, value) {
+.validate_assayDataElementReplace <- function(obj, value) {
+    if (!is.null(value)) {
+        dimvalue <- dim(value)
+        dimobj <- dim(obj)[seq_along(dimvalue)]
+        if (!isTRUE(all.equal(unname(dimvalue), unname(dimobj))))
+            stop("object and replacement value have different dimensions")
+    }
+    
+    if (!is.null(value)) {
+        if (!is.null(dimnames(value))) {
+            ## validate and harmonize dimnames
+            vd <- Map(function(od, vd) {
+                if (is.null(vd))
+                    ## update vd to contain indexes into matrix
+                    od <- seq_along(od)
+                else if (!setequal(od, vd))
+                    stop("object and replacement value dimnames differ")
+                od
+            }, dimnames(obj), dimnames(value))
+            ## re-arrange value to have dimnames in same order as obj
+            value <- do.call(`[`, c(list(value), vd, drop=FALSE))
+        }
+        dimnames(value) <- dimnames(obj)
+    }
+    value
+}
+
+assayDataElementReplace <- function(obj, elt, value, validate=TRUE) {
+    ## 'validate' added later, needs to be last for position matching
+    if (validate)
+        value <- .validate_assayDataElementReplace(obj, value)
+
     storage.mode <- storageMode(obj)
     switch(storageMode(obj),
            "lockedEnvironment" = {
@@ -460,7 +491,10 @@ assayDataElementReplace <- function(obj, elt, value) {
     obj
 }
 
-`assayDataElement<-` <- assayDataElementReplace
+`assayDataElement<-` <- function(obj, elt, ..., value)
+    ## 'value' is always the last argument, but needs to be 3rd for
+    ## assayDataElementReplace
+    assayDataElementReplace(obj, elt, value, ...)
 
 setMethod("phenoData", "eSet", function(object) object@phenoData)
 
